@@ -242,10 +242,12 @@ pyg_type_name (PyObject *self, PyObject *args)
     GType type;
     const gchar *name;
 
+#if 0
     if (PyErr_Warn(PyExc_DeprecationWarning,
 		   "gobject.type_name is deprecated; "
 		   "use GType.name instead"))
         return NULL;
+#endif
     
     if (!PyArg_ParseTuple(args, "O:gobject.type_name", &gtype))
 	return NULL;
@@ -263,18 +265,20 @@ pyg_type_from_name (PyObject *self, PyObject *args)
 {
     const gchar *name;
     GType type;
-
+#if 0
     if (PyErr_Warn(PyExc_DeprecationWarning,
 		   "gobject.type_from_name is deprecated; "
 		   "use GType.from_name instead"))
         return NULL;
-    
+#endif    
     if (!PyArg_ParseTuple(args, "s:gobject.type_from_name", &name))
 	return NULL;
     type = g_type_from_name(name);
     if (type != 0)
 	return pyg_type_wrapper_new(type);
-    PyErr_SetString(PyExc_RuntimeError, "unknown type name");
+    PyErr_Format(PyExc_RuntimeError, "%s: unknown type name: %s",
+		 PyString_AsString(PyObject_Repr((PyObject*)self)),
+		 name);
     return NULL;
 }
 
@@ -283,12 +287,12 @@ pyg_type_parent (PyObject *self, PyObject *args)
 {
     PyObject *gtype;
     GType type, parent;
-
+#if 0
     if (PyErr_Warn(PyExc_DeprecationWarning,
 		   "gobject.type_parent is deprecated; "
 		   "use GType.parent instead"))
         return NULL;
-    
+#endif    
     if (!PyArg_ParseTuple(args, "O:gobject.type_parent", &gtype))
 	return NULL;
     if ((type = pyg_type_from_object(gtype)) == 0)
@@ -305,12 +309,12 @@ pyg_type_is_a (PyObject *self, PyObject *args)
 {
     PyObject *gtype, *gparent;
     GType type, parent;
-
+#if 0
     if (PyErr_Warn(PyExc_DeprecationWarning,
 		   "gobject.type_is_a is deprecated; "
 		   "use GType.is_a instead"))
         return NULL;
-    
+#endif    
     if (!PyArg_ParseTuple(args, "OO:gobject.type_is_a", &gtype, &gparent))
 	return NULL;
     if ((type = pyg_type_from_object(gtype)) == 0)
@@ -326,12 +330,12 @@ pyg_type_children (PyObject *self, PyObject *args)
     PyObject *gtype, *list;
     GType type, *children;
     guint n_children, i;
-
+#if 0
     if (PyErr_Warn(PyExc_DeprecationWarning,
 		   "gobject.type_children is deprecated; "
 		   "use GType.children instead"))
         return NULL;
-    
+#endif    
     if (!PyArg_ParseTuple(args, "O:gobject.type_children", &gtype))
 	return NULL;
     if ((type = pyg_type_from_object(gtype)) == 0)
@@ -357,12 +361,12 @@ pyg_type_interfaces (PyObject *self, PyObject *args)
     PyObject *gtype, *list;
     GType type, *interfaces;
     guint n_interfaces, i;
-
+#if 0
     if (PyErr_Warn(PyExc_DeprecationWarning,
 		   "gobject.type_interfaces is deprecated; "
 		   "use GType.interfaces instead"))
         return NULL;
-    
+#endif    
     if (!PyArg_ParseTuple(args, "O:gobject.type_interfaces", &gtype))
 	return NULL;
     if ((type = pyg_type_from_object(gtype)) == 0)
@@ -925,7 +929,17 @@ add_properties (GType instance_type, PyObject *properties)
 	if (pspec) {
 	    g_object_class_install_property(oclass, 1, pspec);
 	} else {
+            PyObject *type, *value, *traceback;
 	    ret = FALSE;
+            PyErr_Fetch(&type, &value, &traceback);
+            if (PyString_Check(value)) {
+                char msg[256];
+                g_snprintf(msg, 256, "%s (while registering property '%s' for GType '%s')",
+                           PyString_AsString(value), prop_name, g_type_name(instance_type));
+                Py_DECREF(value);
+                value = PyString_FromString(msg);
+            }
+            PyErr_Restore(type, value, traceback);
 	    break;
 	}
     }
@@ -975,7 +989,7 @@ _wrap_pyg_type_register(PyObject *self, PyObject *args)
 {
     PyTypeObject *class;
 
-    if (!PyArg_ParseTuple(args, "O:gobject.type_register", &class))
+    if (!PyArg_ParseTuple(args, "O!:gobject.type_register", &PyType_Type, &class))
 	return NULL;
     if (!PyType_IsSubtype(class, &PyGObject_Type)) {
 	PyErr_SetString(PyExc_TypeError,"argument must be a GObject subclass");
@@ -1320,7 +1334,8 @@ pyg_signal_list_names (PyObject *self, PyObject *args, PyObject *kwargs)
     guint n;
     guint *ids;
     guint i;
-    
+    gpointer iface = NULL;
+
     if (!PyArg_ParseTupleAndKeywords(args, kwargs,
                                      "O:gobject.signal_list_names",
                                      kwlist, &py_itype))
@@ -1339,6 +1354,8 @@ pyg_signal_list_names (PyObject *self, PyObject *args, PyObject *kwargs)
 	PyErr_SetString(PyExc_TypeError,
 			"type must be instantiable or an interface");
 	return NULL;
+    } else {
+        iface = g_type_default_interface_ref(itype);
     }
     
     ids = g_signal_list_ids(itype, &n);
@@ -1352,7 +1369,10 @@ pyg_signal_list_names (PyObject *self, PyObject *args, PyObject *kwargs)
     
     g_free(ids);
     if (class)
-	g_type_class_unref(class);
+	   g_type_class_unref(class);
+    else
+       g_type_default_interface_unref(iface);
+
     return list;
 }
 
@@ -1652,12 +1672,12 @@ pyg_object_new (PyGObject *self, PyObject *args, PyObject *kwargs)
     g_free(params);
     g_type_class_unref(class);
     
-    if (obj)
+    if (obj) {
 	self = (PyGObject *) pygobject_new_full((GObject *)obj, FALSE);
-    else
+        g_object_unref(obj);
+        pygobject_sink(obj);
+    } else
         self = NULL;
-    g_object_unref(obj);
-    pygobject_sink(obj);
 
     return (PyObject *) self;
 }
@@ -2599,6 +2619,20 @@ pyg_set_object_has_new_constructor(GType type)
     g_type_set_qdata(type, pygobject_has_updated_constructor_key, GINT_TO_POINTER(1));
 }
 
+static void
+_log_func(const gchar *log_domain,
+          GLogLevelFlags log_level,
+          const gchar *message,
+          gpointer user_data)
+{
+    PyGILState_STATE state;
+    PyObject* warning = user_data;
+
+    state = pyg_gil_state_ensure();
+    PyErr_Warn(warning, (char *) message);
+    pyg_gil_state_release(state);
+}
+
 /* ----------------- gobject module initialisation -------------- */
 
 struct _PyGObject_Functions pygobject_api_functions = {
@@ -2690,6 +2724,8 @@ initgobject(void)
 {
     PyObject *m, *d, *o, *tuple;
     PyObject *descr;
+    PyObject *warning;
+    
     PyGParamSpec_Type.ob_type = &PyType_Type;
     
     m = Py_InitModule("gobject", pygobject_functions);
@@ -2868,4 +2904,14 @@ initgobject(void)
     pyg_register_gtype_custom(G_TYPE_STRV,
 			      _pyg_strv_from_gvalue,
 			      _pyg_strv_to_gvalue);
+
+    warning = PyErr_NewException("gobject.Warning", PyExc_Warning, NULL);
+    PyDict_SetItemString(d, "Warning", warning);
+    g_log_set_handler("GLib", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING,
+                      _log_func, warning);
+    g_log_set_handler("GLib-GObject", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING,
+                      _log_func, warning);
+    g_log_set_handler("GThread", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING,
+                      _log_func, warning);
+
 }
